/* >15jmain.c  */
/*-----------------------------------
12 June 2014: This version tidies up HistoOnly so that for each k,
              in the plots of all of the parameters of each component,
              all the mu parameter plots are scaled to have the
              same density area. Likewise for all the SD plots, and
              for all the Wt plots.

14 Apr 2014: tauconst commented out

30 Dec 2012: Changed to receive datafilename as an argument

             Has Init.d, 1.par as input and produces

             ParHistos.d as output
             outpdf.d is available, just switch the OUTPUTPDF flag to 1


19 May 2012: input 1.den as well; expects it to be in this folder

8 May 2012: This read the 1.par file made by my version of nmix.f
            outputs parameter histograms in ParHistos.d

            also outputs pdf graph file as outpdf.d

-------------------------------------*/
#include <math.h>
#include <time.h>

#include "threep9.h"

#define GRAPHICSON 0



#if GRAPHICSON
   #include "gengraph.h"
#endif

// OR  ||

#define MAXNUMCOMPS 25 // allows for 24 components
#define MAXNUMINTVLS 500
#define BSIZE 210000
#define TWO 2
#define REMOVEFACTOR 100.0 //factor used for deleting outlying IS obsns WAS 3.0
                          // was 10.0
#define HWIDTH  600
#define VHEIGHT 400
#define YCOVER  1.0

#define OUTPUTPDF 0 // Easyplot output of pdfs
                    // superceded by ParHistos which does the
                    // same thing for Excel
                    // Set to 1 to output this, 0 otherwise
#define OUT6 1
long n;
long akey, bkey;

FILE * fp1; // used for parameter input info
FILE * fp6; // used for parameter histogram info
FILE * fpready; // used to indicate ParHistos.d is ready

#if 0

void sortintoascend1( long n, double* a)

{/*0*/

double dummy;
int i,j;

for (i=1;i<n;i++)
{/*3*/
   for (j=n-1;j>=i;j--)
   {/*4*/
      if (a[j]>a[j+1])
      {/*5*/

         dummy = a[j];
         a[j] = a[j+1];
         a[j+1] = dummy;

      }/*5*/
   }/*4*/
}/*3*/

}/*0*/

void sortintoascend2( long n, double* a, double* b)

{/*0*/

double dummy;
int i,j;

for (i=1;i<n;i++)
{/*3*/
   for (j=n-1;j>=i;j--)
   {/*4*/
      if (a[j]>a[j+1])
      {/*5*/

         dummy = a[j];
         a[j] = a[j+1];
         a[j+1] = dummy;

         dummy = b[j];
         b[j] = b[j+1];
         b[j+1] = dummy;


      }/*5*/
   }/*4*/
}/*3*/

}/*0*/

void sortintoascend3( long n, double* a, double* b, double* c )

{/*0*/

double dummy;
int i,j;

for (i=1;i<n;i++)
{/*3*/
   for (j=n-1;j>=i;j--)
   {/*4*/
      if (a[j]>a[j+1])
      {/*5*/

         dummy = a[j];
         a[j] = a[j+1];
         a[j+1] = dummy;

         dummy = b[j];
         b[j] = b[j+1];
         b[j+1] = dummy;

         dummy = c[j];
         c[j] = c[j+1];
         c[j+1] = dummy;

      }/*5*/
   }/*4*/
}/*3*/

}/*0*/



// ---------- ShellSort -----------

void ShellSortintoAscendingOneCol(
        long NumRows,
        double* X
        )
{
    //  sorts into ascending order

    long Gap;
    long DoneFlag;
    long Icount;
    long Index;
    double Swap;
    long J;
    
    Icount = 0;

    // -- Gap is half the records

    Gap = (long)(NumRows / 2);

    while( Gap >= 1)
    {
        do
        {
            DoneFlag = 1;
            for( Index = 1; Index <= NumRows - Gap; Index++)
            {
                // -- Compare X 1st 1/2 to 2nd 1/2
                if( X [Index] > X [Index + Gap] )
                {
                    // -- Swap Values if 1st > 2nd

                    Swap = X [Index];
                    X [Index] = X [Index + Gap];
                    X [Index + Gap] = Swap;
                    
                    Icount = Icount + 1;
                    DoneFlag = 0;
                }
            }
        }while ( DoneFlag == 0);
        Gap = (long)(Gap / 2);
    }
}                                                    

void ShellSortintoAscendingTwoCols(
        long istt, long iend,
        double *X, double *Y
        )
{
    //  sorts first col into ascending order, keeping other cols in step

    long Gap;
    long DoneFlag;
    long Icount;
    long Index;
    double Swap;
    long J;
    long iBase;
    long NumRows;

    Icount = 0;

    NumRows = iend - istt + 1;

    iBase = istt - 1;

    // -- Gap is half the records

    Gap = (long)(NumRows / 2);

    while( Gap >= 1)
    {
        do
        {
            DoneFlag = 1;
            for( Index = iBase + 1; Index <= iBase + NumRows - Gap; Index++)
            {
                // -- Compare X 1st 1/2 to 2nd 1/2
                if( X [Index] > X [Index + Gap] )
                {
                    // -- Swap Values if 1st > 2nd

                    Swap = X [Index];
                    X [Index] = X [Index + Gap];
                    X [Index + Gap] = Swap;
                    
                    Swap = Y [Index];
                    Y [Index] = Y [Index + Gap];
                    Y [Index + Gap] = Swap;

                    Icount = Icount + 1;
                    DoneFlag = 0;
                }
            }
        }while ( DoneFlag == 0);
        Gap = (long)(Gap / 2);
    }
}                                                    



void ShellSortintoAscendingThreeCols(
        long NumRows,
        double *X, double *Y, double *Z
        )
{
    //  sorts first col into ascending order, keeping other cols in step

    long Gap;
    long DoneFlag;
    long Icount;
    long Index;
    double Swap;
    long J;
    
    Icount = 0;

    // -- Gap is half the records

    Gap = (long)(NumRows / 2);

    while( Gap >= 1)
    {
        do
        {
            DoneFlag = 1;
            for( Index = 1; Index <= NumRows - Gap; Index++)
            {
                // -- Compare X 1st 1/2 to 2nd 1/2
                if( X [Index] > X [Index + Gap] )
                {
                    // -- Swap Values if 1st > 2nd

                    Swap = X [Index];
                    X [Index] = X [Index + Gap];
                    X [Index + Gap] = Swap;
                    
                    Swap = Y [Index];
                    Y [Index] = Y [Index + Gap];
                    Y [Index + Gap] = Swap;

                    Swap = Z [Index];
                    Z [Index] = Z [Index + Gap];
                    Z [Index + Gap] = Swap;

                    Icount = Icount + 1;
                    DoneFlag = 0;
                }
            }
        }while ( DoneFlag == 0);
        Gap = (long)(Gap / 2);
    }
}                                                    


void ShellSortintoDescendingTwoCols(
        long istt, long iend,
        double *X, double *Y
        )
{
    //  sorts first col into ascending order, keeping other cols in step

    long Gap;
    long DoneFlag;
    long Icount;
    long Index;
    double Swap;
    long J;
    long iBase;
    long NumRows;

    Icount = 0;

    NumRows = iend - istt + 1;

    iBase = istt - 1;

    // -- Gap is half the records

    Gap = (long)(NumRows / 2);

    while( Gap >= 1)
    {
        do
        {
            DoneFlag = 1;
            for( Index = iBase + 1; Index <= iBase + NumRows - Gap; Index++)
            {
                // -- Compare X 1st 1/2 to 2nd 1/2
                if( X [Index] < X [Index + Gap] )
                {
                    // -- Swap Values if 1st > 2nd

                    Swap = X [Index];
                    X [Index] = X [Index + Gap];
                    X [Index + Gap] = Swap;
                    
                    Swap = Y [Index];
                    Y [Index] = Y [Index + Gap];
                    Y [Index + Gap] = Swap;

                    Icount = Icount + 1;
                    DoneFlag = 0;
                }
            }
        }while ( DoneFlag == 0);
        Gap = (long)(Gap / 2);
    }
}                                                    


/*====== order data ====*/
void sortintodescend2( long m, long n, double* a, double* b)
/* sorts the segment from m to n inclusive */
{/*0*/

double dummy;
int i,j;

for (i=m;i<n;i++)
{/*3*/
   for (j=n-1;j>=i;j--)
   {/*4*/
      if (a[j]<a[j+1])
      {/*5*/

         dummy = a[j];
         a[j] = a[j+1];
         a[j+1] = dummy;

         dummy = b[j];
         b[j] = b[j+1];
         b[j+1] = dummy;

      }/*5*/
   }/*4*/
}/*3*/

}/*0*/

#endif


/* =================== Histogram Only ========================*/
void HistoOnlyOriginal(
             double xorig[][BSIZE],  /* IN: the points in columns */
             double yorig[],         /* IN: the postprob in a column */
             long numintvls,              /* IN: # of histogram cells */
             double freq[][MAXNUMINTVLS], /* OUT: cell frequencies */
             double hmax[],               /* OUT: max histo height */
             long numpars,                /* IN: # of parameters being done */
             long numpoints,              /* IN: # of points */
             double ProbofK[],          /* incidental info */
             FILE * fopdf                /* output pdf file */

          )

{/*his*/
#if (WATCOM || PC)
static double hist[MAXNUMINTVLS];
static double pk[MAXNUMINTVLS];
static double pdfval[MAXNUMINTVLS];
#endif

char akey;

static long idum;

double histcellw;   /* in original units */
double histcellh;
double histcellwnorm;

long i;
long ialf;
long ik;  /*-subscript of array holding number value of the degree k -*/
long ikderive; /* subscript of array holding derived degree */
long imu;
long isig;
long ixcell;

long j;

static double lbx[40], lby[40];

long nh, nr;
long numcomps;

// long numpdfpts;

double ptot;

static double rtx[40], rty[40];

double x0;
double xa, xaorig;
double xb, xborig;
static double xmax[PSIZE], xmin[PSIZE];
static double xrangeorig[PSIZE];  /* range of xvalues in original units */
double xleft;

double ya, yb;
double yaorig, yborig;
double ybot;
static double ymax[PSIZE];
double ytop;
double ywidth, ywidthm;
double yorigtotal;

for(i=0; i<=numpars+1; i++)
for(j=0; j<MAXNUMINTVLS; j++)
   freq[i][j] = 0.0;

/* ===== there are numcomps rows of graphs ===== */
numcomps = numpars/3;

ywidth = 1.0/(double)numcomps;

/* ======== left graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i-2] =0.01;                      rtx[3*i-2] = 0.33;
   lby[3*i-2] =1.0-((double)i)*ywidth; rty[3*i-2] = lby[3*i-2] + ywidth;
   
}

/* ======== mid graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i-1] =.33;                    rtx[3*i-1] = 0.66;
   lby[3*i-1] =1.0-((double)i)*ywidth; rty[3*i-1] = lby[3*i-1] + ywidth;
   
}

/* ======== right graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i] =0.66;                   rtx[3*i] = .99;
   lby[3*i] =1.0-((double)i)*ywidth; rty[3*i] = lby[3*i] + ywidth;
}

#if GRAPHICSON
   /*---Set screen---*/
   Initgraphics(); /*640x480 16 col*/
#endif

   /*====== HISTOGRAM =====*/


#if GRAPHICSON
   setcolgen(WHITE);
   //   clearscreengen();
#endif

   yorigtotal = 0.0;
   for(i=1;i<=numpoints; i++)
   {
      yorigtotal = yorigtotal + yorig[i];
   }
   if(yorigtotal<= 0.0) printf("No observations to plot\n");

   // printf("In HistOnly: numpoints =%ld\n numpars= %ld\n",numpoints,numpars);
   // printf("yorigtotal = %lf\n", yorigtotal);
   // getch();

   /* ---------------- main calculation ------------------ */
   for(nh=1;nh<=numpars;nh++)  //number of histogram (in this particular k)
   {/*nh*/


      //--- find range ---
      {
      xmax[nh] = -1e20; xmin[nh] = 1e20;

      for(i=1;i<=numpoints;i++)
      {
         if(xorig[nh][i]>xmax[nh]) xmax[nh] = xorig[nh][i];
         if(xorig[nh][i]<xmin[nh]) xmin[nh] = xorig[nh][i];
      }
  
      /*----- range -----*/
      xrangeorig[nh] =  xmax[nh] - xmin[nh];

//         printf("xmin=%lf xmax=%lf\n",xmin[nh],xmax[nh]);

      if(xrangeorig[nh] == 0.) /* - if observations form a single spike - */
      {  /* --- then turn this into a rectangular block --- */
         // printf("!!! xmin=%lf xmax=%lf\n",xmin[nh],xmax[nh]);
         //  getch();
         xmax[nh] += 0.5;
         xmin[nh] -= 0.5;
         xrangeorig[nh] = 1.0;
      }

      /* --- rescale width --- */
      xmin[nh] = xmin[nh] - xrangeorig[nh]/10.0;
      xmax[nh] = xmax[nh] + xrangeorig[nh]/10.0;
      xrangeorig[nh] =  xmax[nh] - xmin[nh];
   }

//      printf("nh=%2ld xmin=%8.3lf xmax=%8.3lf ",nh,xmin[nh],xmax[nh]);
//      printf("Xrange = %8.3lf\n",xrangeorig[nh]);

      // ----- size of histo cells scaled
      //       each cell scaled to have unit area in original units
      
      histcellw = xrangeorig[nh]/(double)numintvls;
      histcellh = 1.0/histcellw; /* cell ht in original units */

      /* ---------- build up frequency counts ------------ */
      for(i=1;i<=numpoints;i++)
      {/*i*/
         /*----- identify and incr the intvl into which the point falls;
                 horiz divided into HRES/histcellw intvls -----*/

         ixcell = 0.5 + (long)( 0.99 * (xorig[nh][i]-xmin[nh]) /histcellw);

         if(ixcell==0) {ixcell = 1;}

         freq[nh][ixcell] += yorig[i]; //-- Here,frequency is a proportion
                                       //  as yorig[i] = 1/numobsns
                                       
      }/*i*/

      /* ------ rescale histobars into pixels ------ 

      on entry each cell has unit height in orig units;
      multiplying by histcellh makes each cell of unit volume, so
      dividing by total num of units makes the total volume unity
      in original units.
      hmax already set at max pdfval on entry at this point
      ------------------------------------------------*/
      
      hmax[nh] = 0.0;

      for(i=1;i<=numintvls;i++)
      {
         hist[i] = freq[nh][i]*histcellh/numpoints;
         if(hist[i]>hmax[nh]) hmax[nh] = hist[i];
      }

      /*----- graph dimensions -----*/
      xleft = 1 + ((nh-1)%3)*HWIDTH/3;
      histcellwnorm = (HWIDTH/3)/(double)numintvls;
      ywidth = VHEIGHT/(pow(numpars, 0.66)+1);
                           // scaled so graph sizes do not change too fast
      ywidthm = YCOVER*ywidth;
      ybot = ywidth*( (long)( (nh-1)/3 ) );
      ytop = ybot + ywidthm;
      yborig = 0.0;

      /*----------------------
      now scale to y-pixel value by reducing by hmax
      -----------------------*/
      for(i=1;i<=numintvls;i++)
      {
         hist[i] = ywidthm * hist[i]/hmax[nh] + ybot;
      }
   

#if GRAPHICSON

      /* ------------- draw the histogram --------------- */

      /*--------- plot axes of frequency histogram graph ----------*/
      setcolgen(WHITE);
      movegen(xleft,ytop); drawgen(xleft+HWIDTH/3,ytop);   /*xaxis-top*/
      movegen(xleft,ybot); drawgen(xleft+HWIDTH/3,ybot);   /*xaxis-bot*/
      movegen(xleft,ytop); drawgen(xleft,ybot);         /*yaxis-l*/
      movegen(xleft+HWIDTH/3,ytop); drawgen(xleft+HWIDTH/3,ybot); /*yaxis-r*/

#endif

#if OUTPUTPDF
      fprintf(fopdf,"******* open graph # %2ld.%2ld *******\n",numcomps,nh);
      fprintf(fopdf,"//pos %4.2lf %4.2lf %4.2lf %4.2lf\n",
                         lbx[nh],lby[nh],rtx[nh],rty[nh]);
      fprintf(fopdf,"/ag \n");
      fprintf(fopdf,"/td \"xy\"  ;define columns \n"); /* define format */
      fprintf(fopdf,"//cwt 2  ;change line weight\n"); /* define line weight */
      fprintf(fopdf,"/sm OFF  ;no marker\n"); /* no marker */
      fprintf(fopdf,"/sd OFF  ;contus line\n"); /* solid line  */

      fprintf(fopdf,"%e\t %e\n", xmin[nh], yborig); /*WAS yborig*/
      fprintf(fopdf,"%e\t %e\n", xmax[nh], yborig); /*WAS yborig*/

      fprintf(fopdf,"//nc   ;end of curve \n");
#endif

#if OUT6
      fprintf(fp6,"%2ld %2ld\n",numcomps,nh); //current k and graph number

//  =============    position of graph ignored
//      fprintf(fp6,"%4.2lf %4.2lf %4.2lf %4.2lf\n",
//                         lbx[nh],lby[nh],rtx[nh],rty[nh]); // posn of graph

#endif


#if GRAPHICSON
      setcolgen(GREEN);
#endif



#if OUTPUTPDF
      fprintf(fopdf,"/td \"xy\"  ;define columns \n"); /* define format */
      fprintf(fopdf,"//cwt 2  ;change line weight\n"); /* define line weight */
      fprintf(fopdf,"/sm OFF  ;no marker\n"); /* no marker */
      fprintf(fopdf,"/sd OFF  ;contus line\n"); /* solid line  */

#endif

#if OUT6
      // ---- this is the number of pairs of points in this histo
      fprintf(fp6,"%ld\n", 2*numintvls+2);
#endif

      for(i=1;i<=numintvls;i++)
      {/*i*/
         ixcell = i;
         xa = histcellwnorm*(ixcell-1) + xleft;
         xb = xa + histcellwnorm;
         xaorig = ixcell * histcellw/0.99 + xmin[nh];
         xborig = xaorig + histcellw;
         ya = hist[ixcell];
         yaorig = hist[i] - ybot;

         yborig = 0.0;


#if GRAPHICSON

         if(i==1) movegen(xa,ybot);
         drawgen(xa,ya);
         drawgen(xb,ya);
         if(i==numintvls) drawgen(xb,ybot);
#endif


#if OUTPUTPDF
         if(i==1) fprintf(fopdf,"%e\t %e\n",xaorig, yborig);
         fprintf(fopdf,"%e\t %e\n",xaorig, yaorig);
         fprintf(fopdf,"%e\t %e\n",xborig, yaorig);
         if(i==numintvls) fprintf(fopdf,"%e\t %e\n",xborig, yborig);
#endif

#if OUT6 // --- point pairs of histo ---
         if(i==1) fprintf(fp6,"%e\t %e\n",xaorig, yborig);
         fprintf(fp6,"%e\t %e\n",xaorig, yaorig);
         fprintf(fp6,"%e\t %e\n",xborig, yaorig);
         if(i==numintvls) fprintf(fp6,"%e\t %e\n",xborig, yborig);
#endif


      }/*i*/

#if OUTPUTPDF
      fprintf(fopdf,"//nc   ;end of curve \n ");
#endif

   /*------------ end of histogram plot -----------*/

   }/*nh*/


#if GRAPHICSON

   getch(); // ----- pause to see plot

   /* ---- close the graphics ----- */
   Closegraphics();

#endif

// moved outside subroutine
// #if OUTPUTPDF
//if(firsttimeflag==-1)
//{
//   fclose(fopdf);
//}
//#endif

}/*his*/
/*======================= end of HistoOnly =========================*/


/* =================== Histogram Only ========================*/
void HistoOnly(
             double xorig[][BSIZE],  /* IN: the points in columns */
             double yorig[],         /* IN: the postprob in a column */
             long numintvls,              /* IN: # of histogram cells */
             double freq[][MAXNUMINTVLS], /* OUT: cell frequencies */
             double hmax[],               /* OUT: max histo height */
             long numpars,                /* IN: # of parameters being done */
             long numobsns,              /* IN: # of obsns */
             double ProbofK[],          /* incidental info */
             FILE * fopdf                /* output pdf file */

          )

{/*his*/
#if (WATCOM || PC)
static double hist[MAXNUMINTVLS];
static double pk[MAXNUMINTVLS];
static double pdfval[MAXNUMINTVLS];
#endif

char akey;

static long idum;

double histcellw;   /* in original units */
double histcellh;
double histcellwnorm;

long i;
long ialf;
long ik;  /*-subscript of array holding number value of the degree k -*/
long ikderive; /* subscript of array holding derived degree */
long imu;
long isig;
long ixcell;

long j;

static double lbx[40], lby[40];

long nh, nr;
long numcomps;

// long numpdfpts;

double ptot;

static double rtx[40], rty[40];

double x0;
double xa, xaorig;
double xb, xborig;
static double xmax[PSIZE], xmin[PSIZE];
static double xrangeorig[PSIZE];  /* range of xvalues in original units */
double xleft;

double ya, yb;
double yaorig, yborig;
double ybot;
static double ymax[PSIZE];
double ytop;
double ywidth, ywidthm;
double yorigtotal;

long partype; // partype identifier 0,1 or 2
double xmaxtype[3];
double xmintype[3];

double ymaxtype[3];

for(i=0; i<=numpars+1; i++)
for(j=0; j<MAXNUMINTVLS; j++)
   freq[i][j] = 0.0;

/* ===== there are numcomps rows of graphs ===== */
numcomps = numpars/3;

ywidth = 1.0/(double)numcomps;

/* ======== left graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i-2] =0.01;                      rtx[3*i-2] = 0.33;
   lby[3*i-2] =1.0-((double)i)*ywidth; rty[3*i-2] = lby[3*i-2] + ywidth;
   
}

/* ======== mid graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i-1] =.33;                    rtx[3*i-1] = 0.66;
   lby[3*i-1] =1.0-((double)i)*ywidth; rty[3*i-1] = lby[3*i-1] + ywidth;
   
}

/* ======== right graph positions (for the outputfile only) ====== */
for(i=1;i<=numcomps;i++)
{
   lbx[3*i] =0.66;                   rtx[3*i] = .99;
   lby[3*i] =1.0-((double)i)*ywidth; rty[3*i] = lby[3*i] + ywidth;
}

#if GRAPHICSON
   /*---Set screen---*/
   Initgraphics(); /*640x480 16 col*/
#endif

   /*====== HISTOGRAM =====*/


#if GRAPHICSON
   setcolgen(WHITE);
   //   clearscreengen();
#endif

   yorigtotal = 0.0;
   for(i=1;i<=numobsns; i++)
   {
      yorigtotal = yorigtotal + yorig[i];
   }
   if(yorigtotal<= 0.0) printf("No observations to plot\n");

   // printf("In HistOnly: numobsns =%ld\n numpars= %ld\n",numobsns,numpars);
   // printf("yorigtotal = %lf\n", yorigtotal);
   // getch();

   /* ---------------- main calculation ------------------ */

   // ===== get overall range for each par tpye
   partype = 0;
   for(i=0;i<=2;i++)
   {
      xmaxtype[i] = -1e20;
      xmintype[i] =  1e20;
   }

   for(nh=1;nh<=numpars;nh++)  //number of histogram (in this particular k)
   {
      if (partype == 0)
      {
         for(i=1;i<=numobsns;i++)
         {
            if(xorig[nh][i]>xmaxtype[0]) xmaxtype[0] = xorig[nh][i];
            if(xorig[nh][i]<xmintype[0]) xmintype[0] = xorig[nh][i];
         }
      }
      if (partype == 1)
      {
         for(i=1;i<=numobsns;i++)
         {
            if(xorig[nh][i]>xmaxtype[1]) xmaxtype[1] = xorig[nh][i];
            if(xorig[nh][i]<xmintype[1]) xmintype[1] = xorig[nh][i];
         }
      }
      if (partype == 2)
      {
         for(i=1;i<=numobsns;i++)
         {
            if(xorig[nh][i]>xmaxtype[2]) xmaxtype[2] = xorig[nh][i];
            if(xorig[nh][i]<xmintype[2]) xmintype[2] = xorig[nh][i];
         }
      }
      partype++;
      if( partype == 3){ partype = 0;}

   }



   // ===== make frequency counts in each histo

   partype = 0;
   for(i=0;i<=2;i++)
   {
      ymaxtype[i] = -1e20;
   }

   for(nh=1;nh<=numpars;nh++)  //number of histogram (in this particular k)
   {/*nh*/

      //--- find range ---

      xmax[nh] = -1e20;
      xmin[nh] =  1e20;

      for(i=1;i<=numobsns;i++)
      {
         if(xorig[nh][i]>xmax[nh]) xmax[nh] = xorig[nh][i];
         if(xorig[nh][i]<xmin[nh]) xmin[nh] = xorig[nh][i];
      }

      if(xmax[nh]-xmin[nh] == 0.) /* - if observations form a single spike - */
      {  /* --- then turn this into a rectangular block --- */
         // printf("!!! xmin=%lf xmax=%lf\n",xmin[nh],xmax[nh]);
         //  getch();
         xmax[nh] += 0.5;
         xmin[nh] -= 0.5;
         xrangeorig[nh] = 1.0;
      }


//      printf("nh=%2ld xmin=%8.3lf xmax=%8.3lf ",nh,xmin[nh],xmax[nh]);
//      printf("Xrange = %8.3lf\n",xrangeorig[nh]);

      // ----- size of histo cells scaled
      //       each cell scaled to have unit area in original units
      
      histcellw = (xmaxtype[partype]-xmintype[partype])/(double)numintvls;
      histcellh = 1.0/histcellw; /* cell ht in original units */

      /* ---------- build up frequency counts ------------ */
      for(i=1;i<=numobsns;i++)
      {/*i*/
         /*----- identify and incr the intvl into which the point falls;
                 horiz divided into HRES/histcellw intvls -----*/

         ixcell = 0.5 + (long)( 0.99 * (xorig[nh][i]-xmin[nh]) /histcellw);

         if(ixcell==0) ixcell=1;

         freq[nh][ixcell] += yorig[i]; //-- Here,frequency is a proportion
                                       //  as yorig[i] = 1/numobsns
                                       
      }/*i*/

      // ----- get max height of histogram

      for(i=1;i<=numintvls;i++)
      {
         hist[i] = freq[nh][i]*histcellh/numobsns;
         if(hist[i]>ymaxtype[partype]) ymaxtype[partype] = hist[i];
      }



      partype++;
      if(partype==3) {partype=0;}

  }// ---- nh

  partype = 0;
  for(nh=1;nh<=numpars;nh++)  //number of histogram (in this particular k)
  {
      /* ------ rescale histobars into pixels ------ 

      on entry each cell has unit height in orig units;
      multiplying by histcellh makes each cell of unit volume, so
      dividing by total num of units makes the total volume unity
      in original units.
      hmax already set at max pdfval on entry at this point
      ------------------------------------------------*/
      

      /*----- graph dimensions -----*/
      xleft = 1 + ((nh-1)%3)*HWIDTH/3;
      histcellwnorm = (HWIDTH/3)/(double)numintvls;
      ywidth = VHEIGHT/(pow(numpars, 0.66)+1);
                           // scaled so graph sizes do not change too fast
      ywidthm = YCOVER*ywidth;
      ybot = ywidth*( (long)( (nh-1)/3 ) );
      ytop = ybot + ywidthm;
      yborig = 0.0;

      /*----------------------
      now scale to y-pixel value by reducing by hmax
      -----------------------*/

      histcellw = (xmaxtype[partype]-xmintype[partype])/(double)numintvls;
      histcellh = 1.0/histcellw; /* cell ht in original units */

      for(i=1;i<=numintvls;i++)
      {
         hist[i] = freq[nh][i]*histcellh/numobsns;
         hist[i] = ywidthm * hist[i]/ymaxtype[partype] + ybot;
      }
   

#if GRAPHICSON

      /* ------------- draw the histogram --------------- */

      /*--------- plot axes of frequency histogram graph ----------*/
      setcolgen(WHITE);
      movegen(xleft,ytop); drawgen(xleft+HWIDTH/3,ytop);   /*xaxis-top*/
      movegen(xleft,ybot); drawgen(xleft+HWIDTH/3,ybot);   /*xaxis-bot*/
      movegen(xleft,ytop); drawgen(xleft,ybot);         /*yaxis-l*/
      movegen(xleft+HWIDTH/3,ytop); drawgen(xleft+HWIDTH/3,ybot); /*yaxis-r*/

#endif

#if OUTPUTPDF
      fprintf(fopdf,"******* open graph # %2ld.%2ld *******\n",numcomps,nh);
      fprintf(fopdf,"//pos %4.2lf %4.2lf %4.2lf %4.2lf\n",
                         lbx[nh],lby[nh],rtx[nh],rty[nh]);
      fprintf(fopdf,"/ag \n");
      fprintf(fopdf,"/td \"xy\"  ;define columns \n"); /* define format */
      fprintf(fopdf,"//cwt 2  ;change line weight\n"); /* define line weight */
      fprintf(fopdf,"/sm OFF  ;no marker\n"); /* no marker */
      fprintf(fopdf,"/sd OFF  ;contus line\n"); /* solid line  */

      fprintf(fopdf,"%e\t %e\n", xmin[nh], yborig); /*WAS yborig*/
      fprintf(fopdf,"%e\t %e\n", xmax[nh], yborig); /*WAS yborig*/

      fprintf(fopdf,"//nc   ;end of curve \n");
#endif

#if OUT6
      fprintf(fp6,"%2ld %2ld\n",numcomps,nh); //current k and graph number

//  =============    position of graph ignored
//      fprintf(fp6,"%4.2lf %4.2lf %4.2lf %4.2lf\n",
//                         lbx[nh],lby[nh],rtx[nh],rty[nh]); // posn of graph

#endif


#if GRAPHICSON
      setcolgen(GREEN);
#endif



#if OUTPUTPDF
      fprintf(fopdf,"/td \"xy\"  ;define columns \n"); /* define format */
      fprintf(fopdf,"//cwt 2  ;change line weight\n"); /* define line weight */
      fprintf(fopdf,"/sm OFF  ;no marker\n"); /* no marker */
      fprintf(fopdf,"/sd OFF  ;contus line\n"); /* solid line  */

#endif

#if OUT6
      // ---- this is the number of pairs of obsns in this histo
      fprintf(fp6,"%ld\n", 2*numintvls+2);
#endif

      for(i=1;i<=numintvls;i++)
      {/*i*/
         ixcell = i;
         xa = histcellwnorm*(ixcell-1) + xleft;
         xb = xa + histcellwnorm;
         xaorig = ixcell * histcellw/0.99 + xmin[nh];
         xborig = xaorig + histcellw;
         ya = hist[ixcell];
         yaorig = hist[i] - ybot;

         yborig = 0.0;


#if GRAPHICSON

         if(i==1) movegen(xa,ybot);
         drawgen(xa,ya);
         drawgen(xb,ya);
         if(i==numintvls) drawgen(xb,ybot);
#endif


#if OUTPUTPDF
         if(i==1) fprintf(fopdf,"%e\t %e\n",xaorig, yborig);
         fprintf(fopdf,"%e\t %e\n",xaorig, yaorig);
         fprintf(fopdf,"%e\t %e\n",xborig, yaorig);
         if(i==numintvls) fprintf(fopdf,"%e\t %e\n",xborig, yborig);
#endif

#if OUT6 // --- point pairs of histo ---
         if(i==1) fprintf(fp6,"%e\t %e\n",xaorig, yborig);
         fprintf(fp6,"%e\t %e\n",xaorig, yaorig);
         fprintf(fp6,"%e\t %e\n",xborig, yaorig);
         if(i==numintvls) fprintf(fp6,"%e\t %e\n",xborig, yborig);
#endif


      }/*i*/

#if OUTPUTPDF
      fprintf(fopdf,"//nc   ;end of curve \n ");
#endif

   /*------------ end of histogram plot -----------*/

      partype++;
      if(partype==3){partype=0;}

   }/*nh*/


#if GRAPHICSON

   getch(); // ----- pause to see plot

   /* ---- close the graphics ----- */
   Closegraphics();

#endif

// moved outside subroutine
// #if OUTPUTPDF
//if(firsttimeflag==-1)
//{
//   fclose(fopdf);
//}
//#endif

}/*his*/
/*======================= end of HistoOnly =========================*/



int main(int argc, char * argv[])

{/*0*/


static double probK[MAXNUMCOMPS];

long priorKMax;


static double ProbofK[MAXNUMCOMPS];

static NumWithKcomps[PSIZE];

long numcomps0;

static double xorig[PSIZE][BSIZE];  /* IN: the obsns in columns */
static double yorig[BSIZE];  /* IN: the postprobs in a column */
static long numintvls;         /* IN: # of histogram cells */
static double freq[PSIZE ][MAXNUMINTVLS]; /* OUT: cell frequencies */
static double hmax[MAXNUMINTVLS];   /* OUT: max histo height */
static long numpars;            /* IN: # of parameters being done */
static long numobsns;     /* IN: # of obsns */
static long numobsns0;    /* IN: # of obsns for reduced plot */

static double xden[201][8]; // the RG file 1.den is read into this array
                            // xden [1]-[200],[0]-[7]


static long firsttimeflag;      /* IN: Set to 1 if HistoOnly called only once */


static FILE * fopdf;


static long akey;

static long ib;

static long idum;


static double xdum1, xdum2, xdum3;

static double ff;

static long i;
static long j;
static long k;
static long n;

static long numcomps;

static double realtime0;
static double realtime1;


static char DataFileName[20];
static char DataFolderInit[30];
static char DataFolder1Par[30];
static char DataFolderOut6[30]; // old out6 name retained in work variables
static char DataFolderOutPdf[30];


// ====================== Param posterior probs =============================

#if GRAPHICSON
   printf(" See the frequency histograms of the parameter postprobs? (y/b/n)\n");
   akey = getch();
#else
   printf(" Graphics suppressed.\n");
//   getch();
   akey = 'y';
#endif

// printf("argv1=%s\n",argv[1]); //This is the data file name
// getch();

 strcpy(DataFileName, argv[1]);

 strcpy(DataFolderInit,DataFileName);
 strcat(DataFolderInit,"/Init.d");

 strcpy(DataFolder1Par,DataFileName);
 strcat(DataFolder1Par,"/1.Par");

 strcpy(DataFolderOut6,DataFileName);
 strcat(DataFolderOut6,"/ParHistos.d");

 strcpy(DataFolderOutPdf,DataFileName);
 strcat(DataFolderOutPdf,"/OutPdf.d");


 printf("datafile name = %s\n",DataFileName);
 printf("Input file  = %s\n",DataFolderInit);
 printf("Input file  = %s\n",DataFolder1Par);
 printf("Output file = %s\n",DataFolderOut6);

 #if OUTPUTPDF
    printf("datafolder=%s\n",DataFolderOutPdf);
 #endif
 printf("\n");

 // printf("\nPress a key to continue\n");
 // getch();




//if(akey!='n')
//{

// ====================== build up the xorig and yorig arrays

// ======= Plot out the posteriors of the parameters =======

fp1 = fopen(DataFolderInit,"r");

/*== get num-pts ===*/
 fscanf(fp1,"%ld",&n);

/* ============
numintvls = n;

if(numintvls<40)
{
   numintvls = 40;
}
if(numintvls>400)
{
   numintvls = 400;
}
==============*/
numintvls = 400;


/*== get yobsno[]  ===*/
 for (i=1;i<=n;i++)
   {/*2*/
   fscanf(fp1,"%lf\n",&xdum1);
   }/*2*/


 fscanf(fp1,"%lf", &xdum1);     // dataspread

 fscanf(fp1,"%ld", &idum);      // Model


 fscanf(fp1,"%ld", &idum);      // num is
 fscanf(fp1,"%lf", &xdum1);     // min del
//  fscanf(fp1,"%lf", &xdum1);     // tau

 fscanf(fp1,"%lf", &xdum1);     // delta
 fscanf(fp1,"%ld", &idum);      // kmax
 fscanf(fp1,"%lf", &xdum1);     // nu1

 fscanf(fp1,"%lf", &xdum1);     // alpha
 fscanf(fp1,"%lf", &xdum1);     // gee
 fscanf(fp1,"%lf", &xdum1);     // hch

 fscanf(fp1,"%lf", &xdum1);     // stretch
 fscanf(fp1,"%lf", &xdum1);     // groupsize
 fscanf(fp1,"%lf", &xdum1);     // seedIS

 fscanf(fp1,"%ld", &priorKMax); // KMaxMC

 // ---- rest of init.d file not used ----

fclose(fp1);

for(numcomps=1; numcomps<=priorKMax; numcomps++) // for each k-comp model
{

   ib = 0;

   fp1 = fopen(DataFolder1Par,"r");

   fscanf(fp1,"%ld\n",&idum); // ==== this is not used, previously read in    ib = 0;

again:

   while(fscanf(fp1,"%ld",&k)!=EOF)
   {

   
//printf("ff=%lf k=%ld numcomps=%ld\n", ff, k, numcomps);
//getch();

      if(k==numcomps)
      {

         ib++;

         // yorig[ib] set later

         for(j=1; j<=numcomps; j++)
         {
            fscanf(fp1,"%lf %lf %lf", &xorig [3*j-2] [ib],
                                      &xorig [3*j-1] [ib],
                                      &xorig [3*j  ] [ib]
                                    );

//printf("At k=%ld j=%ld ib=%ld mu=%lf sd=%lf wt=%lf\n", k,j,ib,
//                 xorig [3*j-2] [ib], xorig [3*j-1] [ib], xorig [3*j ] [ib]);
//getch();


         }
      }
      else
      {
         for(j=1; j<=k; j++)
         {
            fscanf(fp1,"%lf %lf %lf", &xdum1,
                                      &xdum2,
                                      &xdum3
                                    );
         }
      }

   }

   fclose(fp1);

   numobsns0 = ib;

   printf("at k=%ld components, numsweeps=%ld\n",numcomps, numobsns0);
//   getch();


   for(i=1;i<=numobsns0;i++)
   {
      yorig[i] = 1./(double)numobsns0; // --- vertical height of each obsn
   }
 
   numpars = 3*numcomps;

   if(numcomps==1)
   {
      firsttimeflag = 1;
   }
   else if(numcomps==priorKMax)
   {
      firsttimeflag = -1;
   }
   else
   {
      firsttimeflag = 0;
   }


   #if OUTPUTPDF

      if(numcomps==1)
      {

         fopdf = fopen(DataFolderOutPdf,"w");

      }

   #endif

   #if OUT6

      if(numcomps==1)
      {

         fp6 = fopen(DataFolderOut6,"w");

         fprintf(fp6,"%ld\n",priorKMax);
      }

   #endif

      akey =  'y';
      if(numobsns0>0)  // -- output marginal posteriors of params of each component
      {

      HistoOnly(
              xorig,     /* IN: the obsns in columns */
              yorig,     /* IN: the prob vals in one column */
              numintvls, /* IN: # of histogram cells */
              freq,      /* OUT: cell frequencies */
              hmax,      /* OUT: max histo height */
              numpars,   /* IN: # of parameters being plotted */
              numobsns0, /* IN: # of obsns */
              ProbofK,   /* IN: frequency histo of prob of K */
              fopdf /* IN/OUT: */
            );
      }


#if OUTPUTPDF

   if(numcomps==priorKMax)
   {
      fclose(fopdf);
   }

#endif

#if OUT6

   if(numcomps==priorKMax)
   {
      fclose(fp6);
   }

#endif



}/* numcomps */



//printf("\n At Pause press key");
//getch();


goto EndLabel;

// ==== Diagnostic read of 1.den, which needs to be copied to the
// threepi folder

// =============== read in and process 1.den ===================

//fp1 = fopen("1.den","r");

//for(j=0; j<8; j++)
//{
//   for(i=1;i<=200;i++)
//   {
//       fscanf(fp1,"%lf", &xden[i][j]);
//   }

//}
//fclose(fp1);


//for(j=0; j<8; j++)
//{
//   printf("Column %ld\n",j);
//   for(i=1;i<=200;i++)
//   {
//       printf("%lf\n", xden[i][j]);
//   }
//   getch();

//}

EndLabel:


printf("\nOutput in file ParHistos.d\n");

printf("Press key to finish\n");

getch();

// === set up a Flagfile to indicate this program has finished
fpready = fopen("ReadyFileFlag.d","w");
ff = 999.999;
fscanf(fpready,"%lf",&ff);
fclose(fpready);


// === now wait till read of file has taken place
printf("\n\n Please Wait\n");

realtime0 = (double) clock();

againlabel:

realtime1 = (double) clock();

if(realtime1<realtime0 + 2000.0) goto againlabel;


}/*0*/

/* =========== end of main ========== */

