/* >c.15tsubs */

/*=====================

14 oct 2014: includes use of delta=0 to return negloglike only
             neglogposterior subroutine

16 april 2014: includes change of sig prior to RG's

15 april 2014: some changes to priorpar[] normal prior for mean of
               normal, extreme and nextreme models

14 april 2014: compatible with other version 10 modules, see tmain10.c for
               changes made to tmain9.c


Notes:
    (i)    param[0] holds the maximized likval.


No graphics calls in these

=====================*/

// OR ||

#include "threep9.h"

extern char * filedatptr;
extern double * yobs;
extern long n;
extern long numcomps;
extern long akey;
extern long effort;
extern long model;
extern long cgedflag;

/*===== neglikval: negative of loglike or spacings as used in nelmead =====*/
double neglikval(param, priorpar)

double * param;
double * priorpar;

{/*0*/
extern long model;
extern long spacingsflag;   /* for choice of function to be maximised */
extern long show;
extern long numevals;
extern long numcomps;

double incompgam(double,double,double,long);

double gammalog(double, long);

double bigphi(double);
double quanc8(double, double, double *);
double fnf(double, double*, long );
double explicitcdf(double, double* );

#if WATCOM
static double potemp[NSIZE];  /* temp array of scaled pars for quanc8 */
static double q[NSIZE];
#else
double potemp[NSIZE];  /* temp array of scaled pars for quanc8 */
double q[NSIZE];
#endif

double btheta;
double cdf1, cdf2;
double cpsi;
double gamma;
double mu;
double profilelik;
double psi;
double segval;
double sexp;
double space;
double spacings;
double sumoflogqs, sumofobsqs;
double ssigma;
double theta;
double upper;
double wmu;
double xn;
double xt;
double yt;
long i;
long ifault=0;
long iflt;

double Groupsize;
/*======================*/
Groupsize = priorpar[11];

xn = n;
profilelik = 0;
spacings   = 0.0;
cdf1       = 0.0;
for (i=1;i<=n;i++)
{/*3*/
   switch(model)
   {/*4*/

   case NORMAL: 
   case EXTREMEV:
   case NEXTREMEV:
   case MODEL:
   case BURR:
   case XMIXTURE:
   case NXMIXTURE:
   case WMIXTURE:
   case NMIXTURE:
   case LMIXTURE:
   case GMIXTURE:
   case IMIXTURE:
      if (spacingsflag==0)
      {
         xt = fnf(yobs[i],param,model);
         if(xt<=0.00000001) xt = 0.00000001;
         profilelik += log( xt );
      }
      else
      {
         cdf2 = cdf1;
         cdf1 = explicitcdf(yobs[i], param);
         space = cdf1 - cdf2;
         if (space<MINSPACE) space = MINSPACE;
         spacings += log(space);
      }
      break;

   }/*4*/ /*end of switch*/

}/*3*/
   
if(spacingsflag==1)
{
   space = 1. - cdf1;
   if (space<MINSPACE) space = MINSPACE;

   spacings += log(space);
   spacings = spacings/(xn+1.);
}

numevals++;
effort += n;

/*
if( (akey=='s') || ( (akey=='t') && (numevals%10==0) ) )
*/

// if( numevals%1000==0 )
//{
//   printf("numevals = %d \n",numevals);
/*=============
   for(i=1;i<=numcomps;i++)
   {
      printf("p[%d]=%lf p[%d]=%lf p[%d]=%lf\n",
              i,param[3*(i-1)+1], i,param[3*(i-1)+2], i,param[3*(i-1)+3]);
   }
   if(spacingsflag==0)
      printf("minus likelihood= %.10lf\n",-profilelik);
   else
      printf("spacings= %.10lf\n",spacings);
   getch();
=============*/
//}

if(spacingsflag==0)
{
   return  -Groupsize * profilelik;
}
else
{
   return  -Groupsize * spacings;
}

}/*0*/

/* ============== NEGLOGPOSTERIOR =============== */

/* ==== 2:   neglogposterior with
		UNIFORM or BETA Type 2 prior for param1
		BETA Type 2 prior for param 2
==== */
double neglogposterior(double * param, double * priorpar)
{

   double gammalog(double, long);

   long i, j, k;

   long iflt;

   double zeta;
   double kappa;
   double ksi;
   double halfnu1, halfnu2, gamma;
   double delta;

   double eta;
   double arr, arr1, arr2;
   double amu, atau;
   double xd;
   long kmax;

   double mu, tau, awt;

   double normconstnormal;
   double normconstbeta2;

   double alpha;
   double gee;
   double hch;

   double normconstsigRG;

   double logprior;

   delta = priorpar[5];

   if(delta<0.0) // immediate return with negloglike if delta==0.0
   {
      // printf("returning with neglik\n");
      // getch();

      return neglikval(param, priorpar);
   }


   zeta      = priorpar[1]; // samprange
   kappa     = priorpar[3]; // inv of stretched var, in uniform prior for par1
   ksi       = priorpar[4]; // sampmidrange, used in prior for par1
   kmax      = priorpar[6]; // not used, use numcomps instead
   halfnu1   = priorpar[7]; // shape param1 of beta distn in prior for par2
   halfnu2   = priorpar[8]; // shape param2 of beta distn in prior for par2
   gamma     = priorpar[9]; // scale param of beta distn  in prior for par2

   normconstnormal = priorpar[12]; // normalising const of normal prior
   normconstbeta2 = priorpar[13]; // normalising const of beta2 prior

   alpha   = priorpar[14]; // main shape par of RG's beta prior
   gee     = priorpar[15]; // second shape par of RG's beta prior
   hch     = priorpar[16]; // scale par of RG's beta prior

   normconstsigRG = priorpar[17]; // normalising const of RG's beta prior


   logprior = 0.0;

   for(j=1; j<=numcomps; j++) // ---- Assemble the variable terms
   {
      mu = param[3*(j-1)+1];
      tau = param[3*(j-1)+2];
      awt = param[3*(j-1)+3];

      if(model==NMIXTURE || model==NXMIXTURE)
      {
         if( tau <= 0.0  || mu == 0.0 )
         {
            logprior = -100000.0; //WAS 100000.0
            return -logprior;      
         }

      }

      if( awt==0)
      {
         logprior = -100000.0; //WAS 100000.0
         return -logprior;
      }
      else
      {
         arr2 = halfnu1/(gamma*halfnu2);
         switch(model)
         {
            case NMIXTURE:
            case XMIXTURE:
            case NXMIXTURE:
               atau = arr2*tau;
               if( atau<=0.00000001) atau = 0.00000001;
               if( awt<=0.00000001) awt = 0.00000001;

               logprior = logprior

                        - 0.5 * pow( (mu - ksi)/zeta, 2.0)

                        + (2.0 * gee - 1.0) * log( atau ) /* === sig term === */
                        - (alpha + gee) * log( 1.0 + hch * atau * atau )

                        + (delta - 1.0)*log(awt);       /* === awt term === */
            break;
            case GMIXTURE:
            case WMIXTURE:
            case LMIXTURE:
            case IMIXTURE:
               arr1 = halfnu1/(kappa*halfnu2);
               amu = arr1*mu;
               if( amu<=0.00000001) amu = 0.00000001;
               atau = arr2*tau;
               if( atau<=0.00000001) atau = 0.00000001;
               if( awt<=0.00000001) awt = 0.00000001;

               logprior = logprior

                        + (halfnu1 - 1.0) * log( amu )  /* === mean term === */
                        - (halfnu1 + halfnu2) * log( 1.0 + amu )

                        + (2.0 * gee - 1.0) * log( atau ) /* === sig term === */
                        - (alpha + gee) * log( 1.0 + hch * atau * atau )

                        + (delta - 1.0)*log(awt);       /* === awt term === */
            break;
         }
      }
   }

   // ----- add the normalising term
   switch(model)
   {
    case NMIXTURE:
    case XMIXTURE:
    case NXMIXTURE:
      logprior = logprior
                + numcomps * normconstnormal           /* === mean term === */

                + numcomps * normconstsigRG            /* --- sig term === */

                + gammalog((numcomps+1.)*delta, iflt)  /* === awt term === */
                - (numcomps+1.)*gammalog(delta, iflt);  
    break;
    case LMIXTURE:
    case GMIXTURE:
    case WMIXTURE:
    case IMIXTURE:
      logprior = logprior
                + numcomps * ( log(arr1) + normconstbeta2 ) /* === mean term ===*/                  + numcomps * normconstsigRG                 /* === sig term ===*/

                + numcomps * normconstsigRG                 /* --- sig term === */

                + gammalog((numcomps+1.)*delta, iflt)       /* === awt term === */
		- (numcomps+1.)*gammalog(delta, iflt);
    break;

   }

   return  neglikval(param, priorpar)  - logprior;

}


/* ******************* */
void spread(x, xdash, n, delta)

double * x, * xdash;
long n;
double delta;

{/*0*/
extern long model; /* for choice of model             */
extern long spacingsflag;   /* for choice of function to be maximised */
extern long show;
extern long numevals;


long i,j,ict;
double ex;
double tee;

ict = 1;
j = 1;
i = 1;

ex = x[1];

again:
// set xdash[j] to next x[.]. eg for first j xdash[1] = x[1]
xdash[j] = ex;

j++;

if(j== n+1) return;

ict--;

tee = ex + delta; // add delta to current x

while( x[i+1] < tee) // ==== count how many x's in next intvl tee
  {
  ict++;
  i++;
  }

if(ict==0) // ==== if no x's in next intvl move ex to next x[i]
 {
 i++;
 ex = x[i];
 ict++;
 goto again;
 }

ex = ex + delta;

goto again;

/*0*/}

/* ============== This spreads repeated values in the x[.] array and
   puts them in xdash. Each set of repeated values is spread evenly over
   a width of size delta. =============== */

void spread2(x, xdash, n, delta)

double * x, * xdash;
long n;
double delta;

{/*0*/
extern long model; /* for choice of model             */
extern long spacingsflag;   /* for choice of function to be maximised */
extern long show;
extern long numevals;


long i, j, jbase;
long ict;

double delx;
double xbase;
double xt;

 ict = 0;
 i = 1;
 while( i <= n)
 {
 
    if(ict==0) // at start of a fresh point
    {
       xbase = x[i];
       jbase = i;
    }

    if( x[i] == xbase )
    { // add point to current set and move i forward
       ict++;
       if(i==n)
       {
          // finish off
          delx = delta/(double)ict; // makes spread across a width delta
          // delx =delta; // this gives spread with points placed delta apart
          for(j=0;j<ict;j++)
          {
             xdash[jbase+j] = xbase - (ict*delx-delx)/2.0 + j * delx;
          }
       }
       i++;
    }
    else // new x[i] has a different value
    {  // ----- do the spread
       delx = delta/(double)ict; // this makes spread across a width delta 
       // delx = delta;
       for(j=0;j<ict;j++)
       {
          xdash[jbase+j] = xbase - (delx*ict-delx)/2.0 + j * delx;
       }
       ict = 0;
    }
 } /*end while*/
}/*0*/

/* ******************************************** */
/*============ standard normal cdf ==================*/
double bigphi(x)
double x;

  {

  extern long model; /* for choice of model             */
  extern long spacingsflag;   /* for choice of function to be maximised */
  extern long show;
  extern long numevals;

  double a;
  double t;
  double z;

  if (x<-15.0) return 0.0;
  if (x> 15.0) return 1.0;

  if (x<0.0) a = -x;
  else a = x;

  t = 1.0/(1.0 + .2316419*a);
  z = .39894228*exp(-.5*x*x);

  t = z*t*(.31938153 +
        t*(-.356563782 +
        t*(1.781477937 +
        t*(-1.821255978 + 1.330274429*t))));

  if (x<0.) return t;
  else return 1-t;

  }

/*======= pdf of standard normal ========*/

double littlephi(x)
double x;

{
   if(x<-14. || x>14.0) return 0.0;
   else return 0.39894228*exp(-x*x/2.0);
}

/*=================fitstat ====================*/
/* === fits Anderson Darling ===== */
void fitstat(
             double* adstat, long cdfavail, double* yobsno,
             long n, double* pohat, double* Dval  )
{
 double explicitcdf(double, double*);
 double quanc8(double, double, double*);
 double astar;
 long i;
 double cdf;
 double x0, xt;
 double segval, upper;

 astar = 0.0;
 for(i=1;i<=n;i++)
 {/*i*/
    if( cdfavail==1 )
    {
       xt = yobsno[i];
       cdf = explicitcdf(xt,pohat);
    }
    else
    {   
       if(i==1)
       {
          xt = x0 + .00000001;
          upper = yobsno[1];
          if(upper<=xt) upper = xt + 0.00000001;
       }
       else
       {
          xt = upper;
          upper = yobsno[i];
          if(upper<=xt) upper = xt + 0.00000001;
       }
       segval =  quanc8(xt,upper,pohat);
       cdf = cdf+segval;
    }

    if(cdf<=0.0) cdf = 0.00001;
    if(cdf>=1.0) cdf = 0.99999;
    astar += ( 2.* i - 1. )*log( cdf )+ ( 2.* ( n - (i-1) ) - 1.)*log(1.-cdf);

    Dval[i] = (double)(i-0.5)/(n)-cdf;
    /*---
    printf("yobsn = %lf cdf = %lf Dval = %lf\n",yobsno[i],cdf,Dval[i]);
    ---*/  

 }/*i*/

 *adstat = -astar/(double)n  - n;
 /*
 printf("Anderson-Darling statistic = %lf\n",*adstat);
 */

}

/* ======== block for identifying region of poorest fit ======= */
/* ======== identified by j0 and j1 ====== */
void addinterval(
                 long narrowflag, long n,
                 double* maxprobsofar,
                 double* maxprobsofar1,
                 double* Dval,
                 long* j0,long* j1,long* j2,long* j3
                )
 {/* block */

#if WATCOM
 static long numinblock[NSIZE];
 static long blocktype[NSIZE];
 static long Dflag[NSIZE];
#else
    long numinblock[NSIZE];
    long blocktype[NSIZE];
    long Dflag[NSIZE];
#endif

 long i,j;
 long icb;
 long jmin,jmax;
 long numoblocks;
 long jstart;
 double dmax,dmin;

 if(narrowflag==0)
 {
    *maxprobsofar=0;
    for(i=1;i<n;i++)
    {
       dmin = Dval[i];
       for(j=i+1;j<=n;j++)
       {
          if(Dval[j]-dmin>*maxprobsofar)
          {
             *j0 = i;
             *j1 = j;
             *maxprobsofar=Dval[j]-dmin;
          }
       }
    }

    *maxprobsofar1=0.0;
    for(i=1;i<n;i++)
    {
       dmin = Dval[i];
       for(j=i+1;j<=n;j++)
       {
          if(Dval[j]-dmin>*maxprobsofar1 && ( (i<*j0 && j<*j0) || (*j1<i && *j1<j) ) )
          {
             *j2 = i;
             *j3 = j;
             *maxprobsofar1=Dval[j]-dmin;
          }
       }
    }
    /*
    printf("j0=%d j1=%d prob=%lf j2=%d j3=%d prob1=%lf\n",
            *j0,*j1,*maxprobsofar,*j2,*j3,*maxprobsofar1);
    */
 }/* not narrow */

if(narrowflag==1)
{
  /* ==== identify blocks of consecutive i where Dval[i]
          all have the same signs ==== */
  for(i=1;i<=n;i++)
  { 
     if(Dval[i]>0.0) Dflag[i] = 1; else Dflag[i] = -1;
     if(i==1) { icb=1;numoblocks = 0;}
     if(i>1 && Dflag[i]==Dflag[i-1])
     {
        icb++;
        if(i==n)
        {/* at end of last block*/
           numinblock[++numoblocks]=icb;
           if(Dflag[i]==1) blocktype[numoblocks] = -1;
           else blocktype[numoblocks] = 1;
        }
     }
     if(i>1 && Dflag[i]!=Dflag[i-1])
     {/* at end of current block */
        numinblock[++numoblocks] = icb;
        if(Dflag[i]==1) blocktype[numoblocks] = -1;
        else blocktype[numoblocks] = 1;
        icb = 1;
        if(i==n)
        {/* this is the last block anyway */
           numinblock[++numoblocks] = icb;
           if(Dflag[i]==1) blocktype[numoblocks] = 1;
           else blocktype[numoblocks] = -1;
        }
     }
  }

  jstart = 0;
  for(i=1;i<=numoblocks;i++)
  {
     printf("%d blktype = %d numinblk = %d\n",i, blocktype[i],numinblock[i]);
     jstart+= numinblock[i];
  }
  printf("jstart = %d\n",jstart);

/*=======this picks negative slope segments NOT correct! but works ok ======
 ========   Not currently in use ============
  jstart = 1;
  for(i=1;i<=numoblocks;i++)
  { 
     if(blocktype[i]==1)
     {
        dmax = -1e10;
        for(j=jstart;j<jstart+numinblock[i];j++)
        {
           if(Dval[j]>dmax) { dmax = Dval[j]; jmax = j;}
        }
        if(i==numoblocks)
        {
            dmin = 0.0;
            jmin = jstart+numinblock[i]-1;
        }
        if(i<numoblocks)
        {
           dmin = 1e10;
           for(j=jstart+numinblock[i];
               j<jstart+numinblock[i]+numinblock[i+1];
               j++)
           {
              if(Dval[j]<dmin) { dmin = Dval[j]; jmin = j;}
           }
        }

        if(dmax-dmin>*maxprobsofar)
        {
           *maxprobsofar = dmax-dmin;
           *j0 = jmax;
           *j1 = jmin;
        }
     printf("jmax=%d jmin=%d dmax=%lf dmin=%lf intvl=%lf j0 =%d j1 = %d\n",
             jmax, jmin, dmax, dmin, dmax-dmin, *j0, *j1); 
     }
     jstart += numinblock[i];
  }    
===============*/

/* ====== This pick positive slope segments, works quite well on its own === */
  *maxprobsofar = 0.0;
  jstart = 1;
  for(i=1;i<=numoblocks;i++)
  { 
     if(blocktype[i]==-1)
     {
        dmin = 1e10;
        for(j=jstart;j<jstart+numinblock[i];j++)
        {
           if(Dval[j]<dmin) { dmin = Dval[j]; jmin = j;}
        }
        if(i==numoblocks)
        {
            dmax = 0.0;
            jmax = jstart+numinblock[i]-1;
        }
        if(i<numoblocks)
        {
           dmax = -1e10;
           for(j=jstart+numinblock[i];
               j<jstart+numinblock[i]+numinblock[i+1];
               j++)
           {
              if(Dval[j]>dmax) { dmax = Dval[j]; jmax = j;}
           }
        }

        if(dmax-dmin>*maxprobsofar)
        {
           *maxprobsofar = dmax-dmin;
           *j0 = jmin;
           *j1 = jmax;
        }
     printf("jmax=%d jmin=%d dmax=%lf dmin=%lf intvl=%lf j0 =%d j1 = %d\n",
             jmax, jmin, dmax, dmin, dmax-dmin, *j0, *j1); 
     }
     jstart += numinblock[i];
  }    
/* ==========*/
}/* narrowflag */
/*
printf("press key\n");
getchgen();
*/
}/* block */

/* ===== fairly accurate estimate of the gamma power parameter ===== */
double gammaalfa(double r)
{
   if(r<=0.5772)
      return (0.500876 + .1648852*r-.0544274*r*r)/r;
   else
      return (8.898919 + 9.059950*r+.9775373*r*r)/
             (r*(17.79728+11.968477*r+r*r));
}

/*======== mles of extreme value distn =======*/
void mlextreme( yobsn, n, param)

 /* (taken from MLEXT) - calculates mles using Newton-Raphson.
    Includes a normalisation of sample values which ensures
    convergence even in small sample cases which are known to
    be difficult if the s.d. is small compared with the mean.

    Uses damped NR update

  */

 double * param;
 double * yobsn;
 long n;

 {/*0*/
  extern long model; /* for choice of model             */
  extern long spacingsflag;   /* for choice of function to be maximised */
  extern long show;
  extern long numevals;


 long i;
 double corect;
 double f, fdts2;
 double mu;
 double sexp;
 double ssigma;
 double sumy, sumysq, syexp, sy2exp;
 double xn;
 double ybar;
 double zz;

 xn = n;
 for (i=1;i<=n;i++)   yobsn[i] = -yobsn[i];

 /* estimate mean and variance of sample using Newton-Raphson */
 sumy = 0.0;
 sumysq = 0.0;
 for (i=1;i<=n;i++)
    {
    sumy += yobsn[i];
    }
 ybar = sumy/xn;
 for (i=1;i<=n;i++)
    {
    sumysq += pow(yobsn[i]-ybar, 2.);
    }
 corect = sqrt( (double)((6.*sumysq)/xn) )/PI;
 /* initial approximation to gamma is (sqrt(6)/pi)*sample s.d. */
 ssigma = corect;

 printf("Initial ssigma = %lf \n",ssigma);

 while (fabs(corect)>0.0001)
    {
    sumy = 0.;
    sexp = 0.;
    syexp = 0.;
    sy2exp = 0.;

    for (i=1;i<=n;i++)
       {
       sumy   += yobsn[i];
       zz      = exp((ybar-yobsn[i])/ssigma);
       sexp   += zz;
       syexp  += yobsn[i]*zz;
       sy2exp += yobsn[i]*yobsn[i]*zz;
       }
    f       = ssigma - sumy/xn + syexp/sexp;
    fdts2   = pow(ssigma,2.) + sy2exp/sexp - syexp*syexp/(sexp*sexp);

    corect  = ssigma*ssigma*f/fdts2;
    ssigma  = ssigma - 0.5*corect; /*-- damped NR --*/
/*
    printf("corect = %lf ssigma = %lf \n",corect,ssigma);
*/
    }

 mu = -ssigma * log(sexp/xn) + ybar;

 mu = -mu;   /* sign changed because opposite e.v. distn fitted */
 for (i=1;i<=n;i++) yobsn[i] = -yobsn[i];

 param[1] = mu;
 param[2] = ssigma;

 }/*0*/


/*============ log of gamma function ===========*/

double gammalog( xvalue, ifault )

double xvalue;
long ifault;

/* algorithm AS245, Applied Stats (1989) vol. 38, no. 2   */

{
  extern long model; /* for choice of model             */
  extern long spacingsflag;   /* for choice of function to be maximised */
  extern long show;
  extern long numevals;

   double alngam;
/* fixed constants  */
   double alr2pi = 9.18938533204673e-1;
   double four = 4.0e0;
   double half = 5.0e-1;

   long idummy = 0;

   double one = 1.0e0;
   double onep5 = 1.5e0;
   double twelve = 1.2e1;
   double zero = 0.0e0;
/* machine-dependent constants  */
   double xlge = 2.8e2;
   double xlgst = 2.0e36;
   double x, x1, x2;

   double y;
   double r1[10], r2[10], r3[10], r4[6];

/* coefficients of rational functions */
r1[1] = -2.66685511495e0;  r1[2] = -2.44387534237e1;
r1[3] = -2.19698958928e1;  r1[4] = 1.11667541262e1;
r1[5] = 3.13060547623e0;   r1[6] = 6.07771387771e-1;
r1[7] = 1.19400905721e1;   r1[8] = 3.14690115749e1;
r1[9] = 1.52346874070e1;
r2[1] = -7.83359299449e1;  r2[2] = -1.42046296688e2;
r2[3] = 1.37519416416e2;   r2[4] = 7.86994924154e1;
r2[5] = 4.16438922228e0;   r2[6] = 4.70668766060e1;
r2[7] = 3.13399215894e2;   r2[8] = 2.63505074721e2;
r2[9] = 4.33400022514e1;
r3[1] = -2.12159572323e5;  r3[2] = 2.30661510616e5;
r3[3] = 2.74647644705e4;   r3[4] = -4.02621119975e4;
r3[5] = -2.29660729780e3;  r3[6] = -1.16328495004e5;
r3[7] = -1.46025937511e5;  r3[8] = -2.42357409629e4;
r3[9] = -5.70691009324e2;
r4[1] = 2.79195317918525e-1;  r4[2] = 4.917317610505968e-1;
r4[3] = 6.92910599291889e-2;  r4[4] = 3.350343815022304e0;
r4[5] = 6.012459259764103e0;

x = xvalue;
alngam = zero;

/* test for valid function algorithm argument  */
ifault = 2;

if (x > xlgst) return alngam;
ifault = 1;
if (x <= zero) return alngam;
ifault = 0;

idummy += ifault;

/* calculation for 0<x<0.5 and 0.5<=x<1.5 combined  */
if (x < onep5)
 {
 if (x < half)
 {
  alngam = -log(x);
  y = x + one;
/* test whether x < machine epsilon  */
  if (y == one) return alngam;
 }
 else
 {
  alngam = zero;
  y = x;
  x = (x - half) - half;
 }
 alngam += x * ((((r1[5] * y + r1[4]) * y + r1[3]) * y
             + r1[2]) * y + r1[1]) / ((((y + r1[9]) * y
             + r1[8]) * y + r1[7]) * y + r1[6]);
 return alngam;
}

/* calculation for 1.5<= x < 4.0  */
if (x < four)
 {
 y = (x - one) - one;
 alngam = y * ((((r2[5] * x + r2[4]) * x + r2[3]) * x
            + r2[2]) * x + r2[1]) / ((((x + r2[9]) * x
            + r2[8]) * x + r2[7]) * x + r2[6]);
 return alngam;
 }

/* calculation for 4.0<= x < 12.0  */
if (x < twelve)
 {
 alngam = ((((r3[5] * x + r3[4]) * x + r3[3]) * x
           + r3[2]) * x + r3[1]) / ((((x + r3[9]) * x
           + r3[8]) * x + r3[7]) * x + r3[6]);
 return alngam;
 }
/* calculation for x >= 12.0  */
  y = log(x);
  alngam = x * (y - one) - half * y + alr2pi;
  if (x > xlge) return alngam;
  x1 = one / x;
  x2 = x1 * x1;
  alngam += x1 * ((r4[3] * x2 + r4[2]) * x2
            + r4[1]) / ((x2 + r4[5]) * x2 + r4[4]);
  return alngam;

 }

/*===== digamma function ==== */
 double fndigam(double x)
 {
    double digam;
    double ber[10];
    long i;
    long in;
    long indx;
    double x2;
    double di, dum;

    ber[1]=.1666667;
    ber[2]=.033333333;
    ber[3]=.02380952;
    ber[4]=.033333333;
    ber[5]=.075757575;
    ber[6]=.25311355311;
    ber[7]=1.16666667;
    ber[8]=7.0921569;

    if(x<.01 || x>5000) printf("digam x = %lf out of range\n",x);
    indx = 0;
    x2 = x;
    for( i = 1; i<= 40; i++ ) 
    {
       if(x2>40) break;
       indx = indx + 1;
       x2 = x2 + 1;
    }
 
    in = 8;
    if ( x2>100 && x2<=200) in = 6;
    if ( x2>200 && x2<=500) in = 4;
    if ( x2>500) in = 3;
    digam = log(x2) -0.5/x2;
    for(i = 1;i<=in;i++)
    {
       di = i*2;
       dum = ber[i]/pow(x2,(double)(i*2)) / di;
       digam = digam + pow( (-1), i) * dum;
    }
    if( indx == 0) return digam;
 
    for (i = 1; i<=indx; i++)
    {
       dum = i-1;
       digam = digam -1/(x+dum);
    }
    return digam;
 }

/*=== trigamma function ==== */
double fntrigam(double x)
{
   double a, b, b2, b4, b6, b8;
   double y, z;
   double trigam;

   a = 1e-4;
   b = 5;

   b2 =  .16666666667;
   b4 = -.03333333333;
   b6 =  .02380952381;
   b8 = b4;

   trigam = 0.0;
   if(x<0) printf("x is negative\n");
   z = x;

   if (z<a) return 1.0/(z*z);  

lab10:
   if (z>=b) goto lab20;
   trigam += 1./(z*z);
   z += 1.0;
   goto lab10;

lab20:
   y = 1.0/(z*z);
   trigam += y/2. + (1. + y*(b2+y*(b4+y*(b6+y*b8))))/z;
   return trigam;
}

/*============== incomplete gamma integral ========*/
double incompgam( x, p, g, ifault)

double x, p, g;
long ifault;

/* Algorithm AS32 JRSS C (1970) Vol. 19
-includes amendments of Rice & Rose (Vol. 34) and of Cran (Vol. 38)
Computes incomplete gamma ratio for positive values of arguments x and p.
g must be supplied and should be equal to ln(gamma(p)).
Ifault = 1 if p<=0 else 2 if x<0 else 0.
Uses series expansion if p>x or x<=1, otherwise a continued fraction
approximation                                                             */

{
  extern long model; /* for choice of model             */
  extern long spacingsflag;   /* for choice of function to be maximised */
  extern long show;
  extern long numevals;


/* define accuracy and initialise */
  double acu = 1.0e-8;
  double a, an, argu, b, dif;
  double factor;
  double gamain;
  double gin = 0.0;
  double oflo = 1.0e30;
  double pn[7];
  double rn;
  double term;
  double uflo = 1.0e-30;
  long i;

/* test for admissibility of arguments  */
  ifault = 0;
  if (p <= 0.0)  ifault = 1;
  if (x <  0.0)  ifault = 2;
  if ((ifault > 0) || (x == 0.0)) goto lab50;
  argu = p * log(x) - x - g;
  if (argu < log(uflo))  { ifault = 3; if(x<p) gin=0.0; else gin=1.0;}

  if (ifault > 0.0) goto lab50;
  factor = exp( argu );
  if ((x > 1.0) && (x >= p)) goto lab30;

/* calculation by series expansion */
  gin = 1.0;
  term = 1.0;
  rn = p;

 lab20:
  rn++;
  term = term *x/rn;
  gin += term;
  if (term > acu) goto lab20;
  gin = gin * factor/p;
  goto lab50;

/* calculation by continued fraction */
 lab30:
  a = 1.0 - p;
  b = a + x + 1.0;
  term = 0.0;
  pn[1] = 1.0;
  pn[2] = x;
  pn[3] = x + 1.0;
  pn[4] = x * b;
  gin = pn[3]/pn[4];
 lab32:
  a += 1.0;
  b += 2.0;
  term += 1.0;
  an = a * term;
  for (i=1;i<3;i++)  pn[i+4] = b * pn[i+2] - an * pn[i];
  if (pn[6] == 0.0) goto lab35;
  rn = pn[5]/pn[6];
  dif = fabs(gin - rn);
  if (dif > acu) goto lab34;
  if (dif <= acu*rn) goto lab42;
 lab34:
  gin = rn;
 lab35:
  for (i=1;i<5;i++)  pn[i] = pn[i+2];
  if (fabs(pn[5]) < oflo) goto lab32;
  for (i=1;i<5;i++)  pn[i] = pn[i]/oflo;
  goto lab32;
 lab42:
  gin = 1.0 - factor*gin;

 lab50:
  gamain = gin;

  return gamain;

 }

/*+++++++++++++++++++++++++++++++++++++++++++++++++++*/
/*========================= swap ===================*/
void swap(x1ptr,x2ptr)
double *x1ptr, *x2ptr;
{
double xt;
xt = *x1ptr; *x1ptr = *x2ptr; *x2ptr = xt;
}



